Benchmark and Evaluation Scripts
================================

These scripts automate the process of evaluating the all-in-one and one-by-one
analysis approaches for a range of family sizes.

Prerequisites
-------------

The `dist.sh` script requires [Mako](www.makotemplates.org) for instantiating
different sized families. You can configure the path to your local Mako
installation in the `dist.sh` script by changing the following line:

    MAKO=mako-render-2.7

Preparing the ProFeat model
---------------------------

The `dist.sh` script expects a Mako template that has a single parameter named
`N`. To parametrize a ProFeat model, add the following header at the top of the
file:

    <%
        n = int(N)
    %>

You can then parametrize the family size, for example:

    const max_processes = ${n};

Workflow
--------

1. Given a ProFeat model template `example.profeat.template` and a ProFeat
   properties file `example.fprops`, the `dist.sh` script generates all
   families and their instances for a range of sizes, e.g.:

        dist.sh example.profeat.template 3 10

   The `example.fprops` file and the template must be located in the same
   directory. Instead of providing a ProFeat properties file, you can also
   provide a Prism properties file `example.props`. In this case, the
   Prism properties file is copied without modification.

   The `dist.sh` script generates a `dist` directory and an `example.tar.gz`
   file containing the generated models. The tarball can be copied over to
   another machine for running the benchmarks. Running the generated model
   distribution does _not_ require ProFeat, only a working Prism installation.

2. The generated distribution contains a `run.sh` script that runs the
   benchmarks using the given Prism engine (hybrid, sparse, mtbdd or explicit):

        run.sh mtbdd

   In case only the number of MTBDD nodes is of interest, the `-b` option can
   be used. This will build all models without analyzing them:

        run.sh -b mtbdd

   Using the `-j` option, you can specify how many cores should be used when
   running Prism in parallel:

        run.sh -j 4 mtbdd

   Both maximal time and memory consumption (in MB) can be specified using
   the -t and -m options, respectively. For example, to run the benchmarks
   with 1000 MB for CUDD and a time limit of 1 hour, run is invoked as
   follows:

        run.sh -m 1000 -t 1h mtbdd

   All additional arguments are passed to Prism.

   The `run.sh` script creates a `logs` directory and a `logs.tar.gz` file
   containing the Prism logs.

3. The `time.sh` and `nodes.sh` scripts extract the analysis times and number
   of MTBDD nodes, respectively. The results are extracted per engine, i.e. if
   the benchmarks have been run with different engines the `time.sh` script
   must be invoked once per engine:

        time.sh logs/ mtbdd
        time.sh logs/ sparse
        nodes.sh logs/

   The results are written to the `results` directory.

4. Optionally, the `plot.sh` script can be used to generate plots of the
   results:

        plot.sh results/

   This creates one `svg` file per plot and a `results.html` document showing
   the plots.

